@@ -15,6 +15,7 @@ import sys
 import nibabel as nb
 import matplotlib.pyplot as plt
 import matplotlib as mpl
+import plotly.graph_objects as go
 import scipy.stats as ss
 from matplotlib.patches import Polygon
 from matplotlib.collections import PatchCollection
@@ -28,9 +29,9 @@ _base_dir = os.path.dirname(os.path.abspath(__file__))
 _surf_dir = os.path.join(_base_dir, 'surfaces')
 
 def affine_transform(
-    x1, 
-    x2, 
-    x3, 
+    x1,
+    x2,
+    x3,
     M
     ):
     """
@@ -181,6 +182,7 @@ def vol_to_surf(
         volumes = [volumes]
 
     # Make a list of the files to be mapped
+    volsize=np.zeros((len(volumes),),dtype=int)
     for i in range(len(volumes)):
         if (type(volumes[i]) is nb.Nifti2Image) or (type(volumes[i]) is nb.Nifti1Image):
             Vols.append(volumes[i])
@@ -193,6 +195,13 @@ def vol_to_surf(
             except:
                 print(f'File {volumes[i]} could not be opened')
                 Vols.append(None)
+        if Vols[-1] is None:
+            volsize[i]=0
+        elif Vols[-1].ndim == 3:
+            volsize[i]=1
+        else: 
+            volsize[i]=Vols[-1].shape[3]
+
 
     if firstGood is None:
         sys.exit('Error: None of the images could be opened.')
@@ -205,8 +214,9 @@ def vol_to_surf(
         indices[i] = ijk.T
 
     # Read the data and map it
-    data = np.zeros((numPoints,num_verts))
-    mapped_data = np.zeros((num_verts,len(Vols)))
+    mapped_data = np.zeros((num_verts,volsize.sum()))
+    index=volsize.cumsum()
+    index=np.insert(index,0,0)
     for v,vol in enumerate(Vols):
         if vol is None:
             pass
@@ -214,18 +224,16 @@ def vol_to_surf(
             X = vol.get_data()
             if ignore_zeros:
                 X[X==0] = np.nan
-            for p in range(numPoints):
-                data[p,:] = X[indices[p,:,0],indices[p,:,1],indices[p,:,2]]
-                outside = (indices[p,:,:]<0).any(axis=1) # These are vertices outside the volume
-                data[p,outside] = np.nan
-
+            data = X[indices[:,:,0],indices[:,:,1],indices[:,:,2]]
+            if data.ndim == 2: 
+                data = data.reshape(data.shape + (1,))
             # Determine the right statistics - if function - call it
             if stats=='nanmean':
-                mapped_data[:,v] = np.nanmean(data,axis=0)
+                mapped_data[:,index[v]:index[v+1]] = np.nanmean(data,axis=0)
             elif stats=='mode':
-                mapped_data[:,v],_ = ss.mode(data,axis=0)
+                mapped_data[:,index[v]:index[v+1]],_ = ss.mode(data,axis=0)
             elif callable(stats):
-                mapped_data[:,v] = stats(data)
+                mapped_data[:,index[v]:index[v+1]] = stats(data)
 
     return mapped_data
 
@@ -372,12 +380,11 @@ def make_label_gifti(
     return gifti
 
 def get_gifti_column_names(gifti):
-    """
-    Returns the column names from a gifti file (*.label.gii or *.func.gii)
+    """Returns the column names from a gifti file (*.label.gii or *.func.gii)
 
     Args:
         gifti (gifti image):
-            Nibabel Gifti image 
+            Nibabel Gifti image
 
     Returns:
         names (list):
@@ -397,12 +404,12 @@ def get_gifti_colortable(gifti,ignore_0=True):
 
     Args:
         gifti (gifti image):
-            Nibabel Gifti image 
+            Nibabel Gifti image
 
     Returns:
         rgba (np.ndarray):
             N x 4 of RGB values
-        
+
         cmap (mpl obj):
             matplotlib colormap
 
@@ -412,12 +419,13 @@ def get_gifti_colortable(gifti,ignore_0=True):
     rgba = np.zeros((len(labels),4))
     for i,label in enumerate(labels):
         rgba[i,] = labels[i].rgba
-    
+
     if ignore_0:
         rgba = rgba[1:]
         labels = labels[1:]
 
     cmap = LinearSegmentedColormap.from_list('mylist', rgba, N=len(rgba))
+    mpl.cm.unregister_cmap("mycolormap")
     mpl.cm.register_cmap("mycolormap", cmap)
 
     return rgba, cmap
@@ -428,7 +436,7 @@ def get_gifti_anatomical_struct(gifti):
 
     Args:
         gifti (gifti image):
-            Nibabel Gifti image 
+            Nibabel Gifti image
 
     Returns:
         anatStruct (string):
@@ -447,7 +455,7 @@ def get_gifti_labels(gifti):
 
     Args:
         gifti (gifti image):
-            Nibabel Gifti image 
+            Nibabel Gifti image
 
     Returns:
         labels (list):
@@ -459,17 +467,17 @@ def get_gifti_labels(gifti):
     return labels
 
 def save_colorbar(
-    gifti, 
+    gifti,
     outpath
     ):
     """plots colorbar for gifti object (*.label.gii)
-        
+
     Args:
         gifti (gifti image):
-            Nibabel Gifti image 
+            Nibabel Gifti image
         outpath (str):
             outpath for colorbar
-    
+
     """
     _, ax = plt.subplots(figsize=(1,10)) # figsize=(1, 10)
 
@@ -479,13 +487,13 @@ def save_colorbar(
     bounds = np.arange(cmap.N + 1)
 
     norm = mpl.colors.BoundaryNorm(bounds, cmap.N)
-    cb3 = mpl.colorbar.ColorbarBase(ax, cmap=cmap.reversed(cmap), 
+    cb3 = mpl.colorbar.ColorbarBase(ax, cmap=cmap.reversed(cmap),
                                     norm=norm,
                                     ticks=bounds,
                                     format='%s',
                                     orientation='vertical',
                                     )
-    cb3.set_ticklabels(labels[::-1])  
+    cb3.set_ticklabels(labels[::-1])
     cb3.ax.tick_params(size=0)
     cb3.set_ticks(bounds+.5)
     cb3.ax.tick_params(axis='y', which='major', labelsize=30)
@@ -493,13 +501,28 @@ def save_colorbar(
     plt.savefig(outpath, bbox_inches='tight', dpi=150)
 
 def plot(
-        data, surf=None, underlay='SUIT.shape.gii',
-        undermap='Greys', underscale=[-1, 0.5], overlay_type='func', threshold=None,
-        cmap=None, label_names=None, cscale=None, borders='borders.txt', alpha=1.0,
-        outputfile=None, render='matplotlib', new_figure=False, colorbar=False, cbar_tick_format="%.2g"
+        data,
+        surf=None,
+        underlay='SUIT.shape.gii',
+        undermap='Greys',
+        underscale=[-0.5, 0.5],
+        overlay_type='func',
+        threshold=None,
+        cmap=None,
+        cscale=None,
+        label_names=None,
+        borders='borders.txt',
+        bordercolor = 'k',
+        bordersize = 2,
+        alpha=1.0,
+        render='matplotlib',
+        hover = 'auto',
+        new_figure=True,
+        colorbar=False,
+        cbar_tick_format="%.2g",
+        backgroundcolor = 'w'
         ):
-    """
-    Visualize cerebellar activity on a flatmap
+    """Visualize cerebellar activity on a flatmap
 
     Args:
         data (np.array, giftiImage, or name of gifti file):
@@ -520,29 +543,35 @@ def plot(
         cmap (str)
             Matplotlib colormap used for overlay (defaults to 'jet' if none given)
         label_names (list)
-            labelnames for .label.gii (default is None)
+            labelnames (default is None - extracts from .label.gii )
         borders (str)
             Full filepath of the borders txt file (default: borders.txt in SUIT pkg)
+        bordercolor (char or matplotlib.color)
+            Color of border - defaults to 'k'
+        bordersize (int)
+            Size of the border points - defaults to 2
         cscale (int array)
             Colorscale [min, max] for the overlay, valid input values from -1 to 1 (default: [overlay.max, overlay.min])
         alpha (float)
             Opacity of the overlay (default: 1)
-        outputfile (str)
-            Name / path to file to save figure (default None)
         render (str)
-            Renderer for graphic display 'matplot' / 'opengl'. Dafault is matplotlib
+            Renderer for graphic display 'matplot' / 'plotly'. Dafault is matplotlib
+        hover (str)
+            When renderer is plotly, it determines what is displayed in the hover label: 'auto', 'value', or None
         new_figure (bool)
-            By default, flatmap.plot renders the color map into matplotlib's current axis. It new_figure is set to True is will create a new figure
+            If False, plot renders into matplotlib's current axis. If True, it creates a new figure (default=True)
         colorbar (bool)
             By default, colorbar is not plotted into matplotlib's current axis (or new figure if new_figure is set to True)
         cbar_tick_format : str, optional
-            Controls how to format the tick labels of the colorbar.
+            Controls how to format the tick labels of the colorbar, and for the hover label.
             Ex: use "%i" to display as integers.
             Default='%.2g' for scientific notation.
 
     Returns:
         ax (matplotlib.axis)
             If render is matplotlib, the function returns the axis
+        fig (plotly.go.Figure)
+            If render is plotly, it returns Figure object
 
     """
     # default directory
@@ -553,6 +582,7 @@ def plot(
     flatsurf = nb.load(surf)
     vertices = flatsurf.darrays[0].data
     faces    = flatsurf.darrays[1].data
+    num_vert = vertices.shape[0]
 
     # Load the overlay if it's a string
     if type(data) is str:
@@ -567,7 +597,7 @@ def plot(
                 label_names = list(labels.get_labels_as_dict().values())
         data_arr = data.darrays[0].data
 
-    # If it's a nd array, copy `data` arr
+    # If it's a nd array, copy data arr
     if type(data) is np.ndarray:
         data_arr = np.copy(data)
 
@@ -592,56 +622,82 @@ def plot(
         for i in range(num_labels):
             label_names.append("label-{:02d}".format(i + idx))
 
-    # map the overlay to the faces
-    overlay_color, cmap, cscale = _map_color(faces=faces, data=data_arr, cscale=cscale, cmap=cmap, threshold=threshold)
+    # decide whether to map to faces
+    if (render=='plotly'):
+        mapfac=None    # Don't map to faces
+    else:
+        mapfac = faces # Map to faces
+
+    # map the overlay to colors:
+    overlay_color, cmap, cscale = _map_color(data=data_arr,
+            faces = mapfac, cscale=cscale,
+            cmap=cmap, threshold=threshold)
 
     # Load underlay and assign color
     if type(underlay) is not np.ndarray:
         if not os.path.isfile(underlay):
             underlay = os.path.join(os.path.join(_surf_dir, underlay))
         underlay = nb.load(underlay).darrays[0].data
-    underlay_color,_,_ = _map_color(faces=faces, data=underlay, cscale=underscale, cmap=undermap)
+    underlay_color,_,_ = _map_color(data=underlay,
+                    faces = mapfac,
+                    cscale=underscale, cmap=undermap)
 
     # Combine underlay and overlay: For Nan overlay, let underlay shine through
-    face_color = underlay_color * (1-alpha) + overlay_color * alpha
-    i = np.isnan(face_color.sum(axis=1))
-    face_color[i,:]=underlay_color[i,:]
-    face_color[i,3]=1.0
+    color = underlay_color * (1-alpha) + overlay_color * alpha
+    i = np.isnan(color.sum(axis=1))
+    color[i,:]=underlay_color[i,:]
+    color[i,3]=1.0
 
     # If present, get the borders
     if borders is not None:
         if not os.path.isfile(borders):
             borders = os.path.join(os.path.join(_surf_dir, borders))
         borders = np.genfromtxt(borders, delimiter=',')
-
     # Render with Matplotlib
-    ax = _render_matplotlib(vertices, faces, face_color, borders, new_figure)
+    if render == 'matplotlib':
+        ax = _render_matplotlib(vertices, faces, color, borders,
+                                bordercolor, bordersize, new_figure,backgroundcolor)
+        # set up colorbar
+        if colorbar:
+            if overlay_type=='label':
+                cbar = _colorbar_label(ax, cmap, cscale, cbar_tick_format, label_names)
+            elif overlay_type=='func':
+                cbar = _colorbar_func(ax, cmap, cscale, cbar_tick_format)
+    elif render == 'plotly':
+        if hover == 'auto':
+            if overlay_type=='func':
+                textlabel = _make_labels(data_arr,cbar_tick_format)
+            if overlay_type=='label':
+                textlabel = _make_labels(data_arr,label_names)
+        if hover == 'value':
+            textlabel = _make_labels(data_arr,cbar_tick_format)
+        elif hover is None:
+            textlabel=None
+
+        ax = _render_plotly(vertices,faces,color,borders,
+                                bordercolor,bordersize, new_figure,
+                                textlabel,backgroundcolor)
 
-    # set up colorbar
-    if colorbar:
-        if overlay_type=='label':
-            cbar = _colorbar_label(ax, cmap, cscale, cbar_tick_format, label_names)
-        elif overlay_type=='func':
-            cbar = _colorbar_func(ax, cmap, cscale, cbar_tick_format)
 
     return ax
 
+
 def _map_color(
-    faces,
     data,
+    faces=None,
     cscale=None,
     cmap=None,
     threshold=None
     ):
-    """
-    Maps data from vertices to faces, scales the values, and
+    """Scales the values, and
     then looks up the RGB values in the color map
+    If faces are provided, maps the data to faces
 
     Args:
-        faces (nd.array)
-            Array of Faces
         data (1d-np-array)
             Numpy Array of values to scale. If integer, if it is not scaled
+        faces (nd.array)
+            Array of Faces, if provided, it maps to faces
         cscale (array like)
             (min,max) of the scaling of the data
         cmap (str, or matplotlib.colors.Colormap)
@@ -652,6 +708,9 @@ def _map_color(
             if one value is given (-inf) is assumed for the lower
 
     """
+    # if scale not given, find it
+    if cscale is None:
+        cscale = np.array([np.nanmin(data), np.nanmax(data)])
 
     # When continuous data, scale and threshold
     if data.dtype.kind == 'f':
@@ -661,30 +720,25 @@ def _map_color(
                 threshold=np.array([-np.inf,threshold])
             data[~np.logical_and(data>threshold[0], data<threshold[1])]=np.nan
 
-        # if scale not given, find it
-        if cscale is None:
-            cscale = np.array([np.nanmin(data), np.nanmax(data)])
-
         # scale the data
         data = ((data - cscale[0]) / (cscale[1] - cscale[0]))
 
-    elif data.dtype.kind == 'i':
-        if cscale is None:
-            cscale = np.array([np.nanmin(data), np.nanmax(data)])
-
     # Map the values from vertices to faces and integrate
-    numFaces = faces.shape[0]
-    face_value = np.zeros((3,numFaces),dtype = data.dtype)
-    for i in range(3):
-        face_value[i,:] = data[faces[:,i]]
+    if faces is not None:
+        numFaces = faces.shape[0]
+        face_value = np.zeros((3,numFaces),dtype = data.dtype)
+        for i in range(3):
+            face_value[i,:] = data[faces[:,i]]
 
-    if data.dtype.kind == 'i':
-        face_value,_ = ss.mode(face_value,axis=0)
-        face_value = face_value.reshape((numFaces,))
+        if data.dtype.kind == 'i':
+            value,_ = ss.mode(face_value,axis=0)
+            value = value.reshape((numFaces,))
+        else:
+            with warnings.catch_warnings():
+                warnings.simplefilter("ignore", category=RuntimeWarning)
+                value = np.nanmean(face_value, axis=0)
     else:
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", category=RuntimeWarning)
-            face_value = np.nanmean(face_value, axis=0)
+        value = data
 
     # Get the color map
     if type(cmap) is str:
@@ -695,16 +749,179 @@ def _map_color(
         cmap = plt.get_cmap('jet')
 
     # Map the color
-    color_data = cmap(face_value)
+    color_data = cmap(value)
 
     # Set missing data 0 for int or NaN for float to NaN
     if data.dtype.kind == 'f':
-        color_data[np.isnan(face_value),:]=np.nan
+        color_data[np.isnan(value),:]=np.nan
     elif data.dtype.kind == 'i':
-        color_data[face_value==0,:]=np.nan
+        color_data[value==0,:]=np.nan
 
     return color_data, cmap, cscale
 
+
+def _render_matplotlib(vertices,faces,face_color,borders,
+                    bordercolor, bordersize, new_figure,backgroundcolor):
+    """
+    Render the data in matplotlib
+
+    Args:
+        vertices (np.ndarray)
+            Array of vertices
+        faces (nd.array)
+            Array of Faces
+        face_color (nd.array)
+            RGBA array of color and alpha of all vertices
+        borders (np.ndarray)
+            default is None
+        bordercolor (char or matplotlib.color)
+            Color of border
+        bordersize (int)
+            Size of the border points
+        new_figure (bool)
+            Create new Figure or render in currrent axis
+
+    Returns:
+        ax (matplotlib.axes)
+            Axis that was used to render the axis
+    """
+    patches = []
+    for i in range(faces.shape[0]):
+        polygon = Polygon(vertices[faces[i],0:2], True)
+        patches.append(polygon)
+    p = PatchCollection(patches)
+    p.set_facecolor(face_color)
+    p.set_edgecolor(face_color)
+    p.set_linewidth(0.5)
+
+    # Get the current axis and plot it
+    if new_figure:
+        fig = plt.figure(figsize=(7,7))
+    ax = plt.gca()
+    ax.add_collection(p)
+    xrang = [np.nanmin(vertices[:,0]),np.nanmax(vertices[:,0])]
+    yrang = [np.nanmin(vertices[:,1]),np.nanmax(vertices[:,1])]
+
+    ax.set_xlim(xrang[0],xrang[1])
+    ax.set_ylim(yrang[0],yrang[1])
+    ax.axis('equal')
+    ax.axis('off')
+    fig=plt.gcf()
+    fig.set_facecolor(backgroundcolor)
+
+    if borders is not None:
+        ax.plot(borders[:,0],borders[:,1],color=bordercolor,
+                marker='.', linestyle=None,
+                markersize=bordersize,linewidth=0)
+
+    return ax
+
+def _render_plotly(vertices,faces,color,borders,
+            bordercolor, bordersize,
+            new_figure, hovertext=None,
+            backgroundcolor='#ffffff'):
+    """
+    Render the data in plotly
+    Args:
+        vertices (np.ndarray)
+            Array of vertices
+        faces (nd.array)
+            Array of Faces
+        face_color (nd.array)
+            RGBA array of color and alpha of all vertices
+        borders (np.ndarray)
+            default is None
+        bordercolor (char or matplotlib.color)
+            Color of border
+        bordersize (int)
+            Size of the border points
+        new_figure (bool)
+            Create new Figure or render in currrent axis
+        hovertext (list of str)
+            Text for hovering for each vertex
+    Returns:
+        ax (matplotlib.axes)
+            Axis that was used to render the axis
+    """
+    # Check whether to color faces or vertices:
+    if color.shape[0]==vertices.shape[0]:
+        vertcolor=color
+        facecolor=None
+    elif color.shape[0]==faces.shape[0]:
+        vertcolor=None
+        facecolor=color
+    else:
+        raise(NameError('Color data not the correct shape'))
+
+    if hovertext is None:
+        hi = 'skip'
+    else:
+        hi = 'text'
+    traces = []
+    colorbar = dict(exponentformat='none',tickformat='%.2f')
+
+    traces.append(go.Mesh3d(x=vertices[:, 0], y=vertices[:, 1], z=vertices[:, 2],
+                    i=faces[:, 0], j=faces[:, 1], k=faces[:, 2],
+                    facecolor = facecolor,
+                    vertexcolor = vertcolor,
+                    lightposition=dict(x=0, y=0, z=2.5),
+                    text = hovertext,
+                    hoverinfo=hi))
+    if borders is not None:
+        bordercolor=_color_matplotlib_to_plotly(bordercolor)
+        traces.append(go.Scatter3d(
+                x=borders[:,0],
+                y=borders[:,1],
+                z=np.ones((borders.shape[0],))*0.01,
+                marker = dict(
+                    color=bordercolor,
+                    size=bordersize,
+                    symbol='circle'),
+                mode='markers',
+                hoverinfo=None))
+
+
+    camera = dict(
+        up=dict(x=0, y=1, z=0),
+        center=dict(x=0, y=0, z=0),
+        eye=dict(x=0, y=0, z=1.1)
+    )
+    axis_dict= dict(visible=False,
+        showbackground=False,
+        showline=False,
+        showgrid=False,
+        showspikes=False,
+        showticklabels=False,
+        title=None,
+        range=[-110,110])
+    scene = dict(xaxis=axis_dict,
+                yaxis=axis_dict,
+                zaxis=axis_dict,
+                aspectmode= 'cube')
+                # aspectratio=dict(x=1, y=1, z=0.1))
+    backgroundcolor=_color_matplotlib_to_plotly(backgroundcolor)
+    fig = go.Figure(data=traces,)
+    fig.update_layout(scene_camera=camera,
+                dragmode=False,
+                margin=dict(r=0, l=0, b=0, t=0),
+                scene = scene,
+                width=400,
+                height=400,
+                paper_bgcolor=backgroundcolor
+                )
+    return fig
+
+def _make_labels(data,labelstr):
+    numvert=data.shape[0]
+    labels = np.empty((data.shape[0],),dtype=np.object)
+    if type(labelstr) is str:
+        for i in range(numvert):
+            labels[i]=f"{data[i]:{labelstr[1:]}}"
+    else:
+        for i in range(numvert):
+            labels[i]=labelstr[data[i]]
+    return labels
+
 def _colorbar_label(
     ax,
     cmap,
@@ -796,50 +1013,18 @@ def _colorbar_func(
 
     return cbar
 
-def _render_matplotlib(vertices,faces,face_color,borders,new_figure):
-    """
-    Render the data in matplotlib: This is segmented to allow for openGL renderer
+
+def _color_matplotlib_to_plotly(color):
+    """Transforms Matplotlib color string to
+    plotly/html hexadecimal representation
 
     Args:
-        vertices (np.ndarray)
-            Array of vertices
-        faces (nd.array)
-            Array of Faces
-        face_color (nd.array)
-            RGBA array of color and alpha of all vertices
-        borders (np.ndarray)
-            default is None
-        new_figure (bool)
-            Create new Figure or render in currrent axis
-
+        color (str): color string
     Returns:
-        ax (matplotlib.axes)
-            Axis that was used to render the axis
+        colorhex (str): web-based color, e.g., '#000000'
     """
-    patches = []
-    for i in range(faces.shape[0]):
-        polygon = Polygon(vertices[faces[i],0:2], True)
-        patches.append(polygon)
-    p = PatchCollection(patches)
-    p.set_facecolor(face_color)
-    p.set_linewidth(0.0)
-
-    # Get the current axis and plot it
-    if new_figure:
-        fig = plt.figure(figsize=(7,7))
-    ax = plt.gca()
-    ax.add_collection(p)
-    xrang = [np.nanmin(vertices[:,0]),np.nanmax(vertices[:,0])]
-    yrang = [np.nanmin(vertices[:,1]),np.nanmax(vertices[:,1])]
-
-    ax.set_xlim(xrang[0],xrang[1])
-    ax.set_ylim(yrang[0],yrang[1])
-    ax.axis('equal')
-    ax.axis('off')
-
-    if borders is not None:
-        ax.plot(borders[:,0],borders[:,1],color='k',
-                marker='.', linestyle=None,
-                markersize=2,linewidth=0)
-    return ax
-
+    if color=='k':
+        color='#000000'
+    if color=='w':
+        color='#ffffff'
+    return color
\ No newline at end of file
